'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.initProxy = undefined;

var _config = require('vue-template-compiler/lib/core/config');

var _config2 = _interopRequireDefault(_config);

var _index = require('../util/index');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* not type checking this file because flow doesn't play well with Proxy */

var initProxy = void 0;

if (process.env.NODE_ENV !== 'production') {
  var allowedGlobals = (0, _index.makeMap)('Infinity,undefined,NaN,isFinite,isNaN,' + 'parseFloat,parseInt,decodeURI,decodeURIComponent,encodeURI,encodeURIComponent,' + 'Math,Number,Date,Array,Object,Boolean,String,RegExp,Map,Set,JSON,Intl,' + 'require' // for Webpack/Browserify
  );

  var warnNonPresent = function warnNonPresent(target, key) {
    (0, _index.warn)('Property or method "' + key + '" is not defined on the instance but ' + 'referenced during render. Make sure to declare reactive data ' + 'properties in the data option.', target);
  };

  var hasProxy = typeof Proxy !== 'undefined' && Proxy.toString().match(/native code/);

  if (hasProxy) {
    var isBuiltInModifier = (0, _index.makeMap)('stop,prevent,self,ctrl,shift,alt,meta');
    _config2.default.keyCodes = new Proxy(_config2.default.keyCodes, {
      set: function set(target, key, value) {
        if (isBuiltInModifier(key)) {
          (0, _index.warn)('Avoid overwriting built-in modifier in config.keyCodes: .' + key);
          return false;
        } else {
          target[key] = value;
          return true;
        }
      }
    });
  }

  var hasHandler = {
    has: function has(target, key) {
      var has = key in target;
      var isAllowed = allowedGlobals(key) || key.charAt(0) === '_';
      if (!has && !isAllowed) {
        warnNonPresent(target, key);
      }
      return has || !isAllowed;
    }
  };

  var getHandler = {
    get: function get(target, key) {
      if (typeof key === 'string' && !(key in target)) {
        warnNonPresent(target, key);
      }
      return target[key];
    }
  };

  exports.initProxy = initProxy = function initProxy(vm) {
    if (hasProxy) {
      // determine which proxy handler to use
      var options = vm.$options;
      var handlers = options.render && options.render._withStripped ? getHandler : hasHandler;
      vm._renderProxy = new Proxy(vm, handlers);
    } else {
      vm._renderProxy = vm;
    }
  };
}

exports.initProxy = initProxy;