'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.initRender = initRender;
exports.renderMixin = renderMixin;

var _index = require('../util/index');

var _vnode = require('../vdom/vnode');

var _vnode2 = _interopRequireDefault(_vnode);

var _lifecycle = require('./lifecycle');

var _createElement = require('../vdom/create-element');

var _renderList = require('./render-helpers/render-list');

var _renderSlot = require('./render-helpers/render-slot');

var _resolveFilter = require('./render-helpers/resolve-filter');

var _checkKeycodes = require('./render-helpers/check-keycodes');

var _bindObjectProps = require('./render-helpers/bind-object-props');

var _renderStatic = require('./render-helpers/render-static');

var _bindObjectListeners = require('./render-helpers/bind-object-listeners');

var _resolveSlots = require('./render-helpers/resolve-slots');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function initRender(vm) {
  vm._vnode = null; // the root of the child tree
  vm._staticTrees = null;
  var parentVnode = vm.$vnode = vm.$options._parentVnode; // the placeholder node in parent tree
  var renderContext = parentVnode && parentVnode.context;
  vm.$slots = (0, _resolveSlots.resolveSlots)(vm.$options._renderChildren, renderContext);
  vm.$scopedSlots = _index.emptyObject;
  // bind the createElement fn to this instance
  // so that we get proper render context inside it.
  // args order: tag, data, children, normalizationType, alwaysNormalize
  // internal version is used by render functions compiled from templates
  vm._c = function (a, b, c, d) {
    return (0, _createElement.createElement)(vm, a, b, c, d, false);
  };
  // normalization is always applied for the public version, used in
  // user-written render functions.
  vm.$createElement = function (a, b, c, d) {
    return (0, _createElement.createElement)(vm, a, b, c, d, true);
  };

  // $attrs & $listeners are exposed for easier HOC creation.
  // they need to be reactive so that HOCs using them are always updated
  var parentData = parentVnode && parentVnode.data;
  /* istanbul ignore else */
  if (process.env.NODE_ENV !== 'production') {
    (0, _index.defineReactive)(vm, '$attrs', parentData && parentData.attrs, function () {
      !_lifecycle.isUpdatingChildComponent && (0, _index.warn)('$attrs is readonly.', vm);
    }, true);
    (0, _index.defineReactive)(vm, '$listeners', vm.$options._parentListeners, function () {
      !_lifecycle.isUpdatingChildComponent && (0, _index.warn)('$listeners is readonly.', vm);
    }, true);
  } else {
    (0, _index.defineReactive)(vm, '$attrs', parentData && parentData.attrs, null, true);
    (0, _index.defineReactive)(vm, '$listeners', vm.$options._parentListeners, null, true);
  }
}

function renderMixin(Vue) {
  Vue.prototype.$nextTick = function (fn) {
    return (0, _index.nextTick)(fn, this);
  };

  Vue.prototype._render = function () {
    var vm = this;
    var _vm$$options = vm.$options,
        render = _vm$$options.render,
        staticRenderFns = _vm$$options.staticRenderFns,
        _parentVnode = _vm$$options._parentVnode;


    if (vm._isMounted) {
      // clone slot nodes on re-renders
      for (var key in vm.$slots) {
        vm.$slots[key] = (0, _vnode.cloneVNodes)(vm.$slots[key]);
      }
    }

    vm.$scopedSlots = _parentVnode && _parentVnode.data.scopedSlots || _index.emptyObject;

    if (staticRenderFns && !vm._staticTrees) {
      vm._staticTrees = [];
    }
    // set parent vnode. this allows render functions to have access
    // to the data on the placeholder node.
    vm.$vnode = _parentVnode;
    // render self
    var vnode = void 0;
    try {
      vnode = render.call(vm._renderProxy, vm.$createElement);
    } catch (e) {
      (0, _index.handleError)(e, vm, 'render function');
      // return error render result,
      // or previous vnode to prevent render error causing blank component
      /* istanbul ignore else */
      if (process.env.NODE_ENV !== 'production') {
        vnode = vm.$options.renderError ? vm.$options.renderError.call(vm._renderProxy, vm.$createElement, e) : vm._vnode;
      } else {
        vnode = vm._vnode;
      }
    }
    // return empty vnode in case the render function errored out
    if (!(vnode instanceof _vnode2.default)) {
      if (process.env.NODE_ENV !== 'production' && Array.isArray(vnode)) {
        (0, _index.warn)('Multiple root nodes returned from render function. Render function ' + 'should return a single root node.', vm);
      }
      vnode = (0, _vnode.createEmptyVNode)();
    }
    // set parent
    vnode.parent = _parentVnode;
    return vnode;
  };

  // internal render helpers.
  // these are exposed on the instance prototype to reduce generated render
  // code size.
  Vue.prototype._o = _renderStatic.markOnce;
  Vue.prototype._n = _index.toNumber;
  Vue.prototype._s = _index.toString;
  Vue.prototype._l = _renderList.renderList;
  Vue.prototype._t = _renderSlot.renderSlot;
  Vue.prototype._q = _index.looseEqual;
  Vue.prototype._i = _index.looseIndexOf;
  Vue.prototype._m = _renderStatic.renderStatic;
  Vue.prototype._f = _resolveFilter.resolveFilter;
  Vue.prototype._k = _checkKeycodes.checkKeyCodes;
  Vue.prototype._b = _bindObjectProps.bindObjectProps;
  Vue.prototype._v = _vnode.createTextVNode;
  Vue.prototype._e = _vnode.createEmptyVNode;
  Vue.prototype._u = _resolveSlots.resolveScopedSlots;
  Vue.prototype._g = _bindObjectListeners.bindObjectListeners;
}