'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createFunctionalComponent = createFunctionalComponent;

var _vnode = require('./vnode');

var _vnode2 = _interopRequireDefault(_vnode);

var _createElement = require('./create-element');

var _inject = require('../instance/inject');

var _resolveSlots = require('../instance/render-helpers/resolve-slots');

var _index = require('../util/index');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createFunctionalComponent(Ctor, propsData, data, context, children) {
  var props = {};
  var propOptions = Ctor.options.props;
  if ((0, _index.isDef)(propOptions)) {
    for (var key in propOptions) {
      props[key] = (0, _index.validateProp)(key, propOptions, propsData || {});
    }
  } else {
    if ((0, _index.isDef)(data.attrs)) mergeProps(props, data.attrs);
    if ((0, _index.isDef)(data.props)) mergeProps(props, data.props);
  }
  // ensure the createElement function in functional components
  // gets a unique context - this is necessary for correct named slot check
  var _context = Object.create(context);
  var h = function h(a, b, c, d) {
    return (0, _createElement.createElement)(_context, a, b, c, d, true);
  };
  var vnode = Ctor.options.render.call(null, h, {
    data: data,
    props: props,
    children: children,
    parent: context,
    listeners: data.on || {},
    injections: (0, _inject.resolveInject)(Ctor.options.inject, context),
    slots: function slots() {
      return (0, _resolveSlots.resolveSlots)(children, context);
    }
  });
  if (vnode instanceof _vnode2.default) {
    vnode.functionalContext = context;
    vnode.functionalOptions = Ctor.options;
    if (data.slot) {
      (vnode.data || (vnode.data = {})).slot = data.slot;
    }
  }
  return vnode;
}

function mergeProps(to, from) {
  for (var key in from) {
    to[(0, _index.camelize)(key)] = from[key];
  }
}