'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _transition = require('../modules/transition');

// recursively search for possible transition defined inside the component root
function locateNode(vnode) {
  return vnode.componentInstance && (!vnode.data || !vnode.data.transition) ? locateNode(vnode.componentInstance._vnode) : vnode;
}

exports.default = {
  bind: function bind(el, _ref, vnode) {
    var value = _ref.value;

    vnode = locateNode(vnode);
    var transition = vnode.data && vnode.data.transition;
    var originalDisplay = el.__vOriginalDisplay = el.style.display === 'none' ? '' : el.style.display;
    if (value && transition) {
      vnode.data.show = true;
      (0, _transition.enter)(vnode, function () {
        el.style.display = originalDisplay;
      });
    } else {
      el.style.display = value ? originalDisplay : 'none';
    }
  },
  update: function update(el, _ref2, vnode) {
    var value = _ref2.value,
        oldValue = _ref2.oldValue;

    /* istanbul ignore if */
    if (value === oldValue) return;
    vnode = locateNode(vnode);
    var transition = vnode.data && vnode.data.transition;
    if (transition) {
      vnode.data.show = true;
      if (value) {
        (0, _transition.enter)(vnode, function () {
          el.style.display = el.__vOriginalDisplay;
        });
      } else {
        (0, _transition.leave)(vnode, function () {
          el.style.display = 'none';
        });
      }
    } else {
      el.style.display = value ? el.__vOriginalDisplay : 'none';
    }
  },
  unbind: function unbind(el, binding, vnode, oldVnode, isDestroy) {
    if (!isDestroy) {
      el.style.display = el.__vOriginalDisplay;
    }
  }
};