'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _util = require('vue-template-compiler/lib/shared/util');

var _index = require('vue-template-compiler/lib/core/vdom/helpers/index');

var _env = require('vue-template-compiler/lib/core/util/env');

var _model = require('vue-template-compiler/lib/platforms/web/compiler/directives/model');

// normalize v-model event tokens that can only be determined at runtime.
// it's important to place the event as the first in the array because
// the whole point is ensuring the v-model callback gets called before
// user-attached handlers.
function normalizeEvents(on) {
  var event = void 0;
  /* istanbul ignore if */
  if ((0, _util.isDef)(on[_model.RANGE_TOKEN])) {
    // IE input[type=range] only supports `change` event
    event = _env.isIE ? 'change' : 'input';
    on[event] = [].concat(on[_model.RANGE_TOKEN], on[event] || []);
    delete on[_model.RANGE_TOKEN];
  }
  if ((0, _util.isDef)(on[_model.CHECKBOX_RADIO_TOKEN])) {
    // Chrome fires microtasks in between click/change, leads to #4521
    event = _env.isChrome ? 'click' : 'change';
    on[event] = [].concat(on[_model.CHECKBOX_RADIO_TOKEN], on[event] || []);
    delete on[_model.CHECKBOX_RADIO_TOKEN];
  }
}

var target = void 0;

function add(event, _handler, once, capture, passive) {
  if (once) {
    var oldHandler = _handler;
    var _target = target; // save current target element in closure
    _handler = function handler(ev) {
      var res = arguments.length === 1 ? oldHandler(ev) : oldHandler.apply(null, arguments);
      if (res !== null) {
        remove(event, _handler, capture, _target);
      }
    };
  }
  target.addEventListener(event, _handler, _env.supportsPassive ? { capture: capture, passive: passive } : capture);
}

function remove(event, handler, capture, _target) {
  (_target || target).removeEventListener(event, handler, capture);
}

function updateDOMListeners(oldVnode, vnode) {
  if ((0, _util.isUndef)(oldVnode.data.on) && (0, _util.isUndef)(vnode.data.on)) {
    return;
  }
  var on = vnode.data.on || {};
  var oldOn = oldVnode.data.on || {};
  target = vnode.elm;
  normalizeEvents(on);
  (0, _index.updateListeners)(on, oldOn, add, remove, vnode.context);
}

exports.default = {
  create: updateDOMListeners,
  update: updateDOMListeners
};