'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _style = require('vue-template-compiler/lib/platforms/web/util/style');

var _util = require('vue-template-compiler/lib/shared/util');

var cssVarRE = /^--/;
var importantRE = /\s*!important$/;
var setProp = function setProp(el, name, val) {
  /* istanbul ignore if */
  if (cssVarRE.test(name)) {
    el.style.setProperty(name, val);
  } else if (importantRE.test(val)) {
    el.style.setProperty(name, val.replace(importantRE, ''), 'important');
  } else {
    var normalizedName = normalize(name);
    if (Array.isArray(val)) {
      // Support values array created by autoprefixer, e.g.
      // {display: ["-webkit-box", "-ms-flexbox", "flex"]}
      // Set them one by one, and the browser will only set those it can recognize
      for (var i = 0, len = val.length; i < len; i++) {
        el.style[normalizedName] = val[i];
      }
    } else {
      el.style[normalizedName] = val;
    }
  }
};

var vendorNames = ['Webkit', 'Moz', 'ms'];

var emptyStyle = void 0;
var normalize = (0, _util.cached)(function (prop) {
  emptyStyle = emptyStyle || document.createElement('div').style;
  prop = (0, _util.camelize)(prop);
  if (prop !== 'filter' && prop in emptyStyle) {
    return prop;
  }
  var capName = prop.charAt(0).toUpperCase() + prop.slice(1);
  for (var i = 0; i < vendorNames.length; i++) {
    var name = vendorNames[i] + capName;
    if (name in emptyStyle) {
      return name;
    }
  }
});

function updateStyle(oldVnode, vnode) {
  var data = vnode.data;
  var oldData = oldVnode.data;

  if ((0, _util.isUndef)(data.staticStyle) && (0, _util.isUndef)(data.style) && (0, _util.isUndef)(oldData.staticStyle) && (0, _util.isUndef)(oldData.style)) {
    return;
  }

  var cur = void 0,
      name = void 0;
  var el = vnode.elm;
  var oldStaticStyle = oldData.staticStyle;
  var oldStyleBinding = oldData.normalizedStyle || oldData.style || {};

  // if static style exists, stylebinding already merged into it when doing normalizeStyleData
  var oldStyle = oldStaticStyle || oldStyleBinding;

  var style = (0, _style.normalizeStyleBinding)(vnode.data.style) || {};

  // store normalized style under a different key for next diff
  // make sure to clone it if it's reactive, since the user likley wants
  // to mutate it.
  vnode.data.normalizedStyle = (0, _util.isDef)(style.__ob__) ? (0, _util.extend)({}, style) : style;

  var newStyle = (0, _style.getStyle)(vnode, true);

  for (name in oldStyle) {
    if ((0, _util.isUndef)(newStyle[name])) {
      setProp(el, name, '');
    }
  }
  for (name in newStyle) {
    cur = newStyle[name];
    if (cur !== oldStyle[name]) {
      // ie9 setting to null has no effect, must use empty string
      setProp(el, name, cur == null ? '' : cur);
    }
  }
}

exports.default = {
  create: updateStyle,
  update: updateStyle
};