'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _util = require('vue-template-compiler/lib/shared/util');

var _textParser = require('vue-template-compiler/lib/compiler/parser/text-parser');

var _helpers = require('vue-template-compiler/lib/compiler/helpers');

var normalize = (0, _util.cached)(_util.camelize);

function transformNode(el, options) {
  var warn = options.warn || _helpers.baseWarn;
  var staticStyle = (0, _helpers.getAndRemoveAttr)(el, 'style');

  var _parseStaticStyle = parseStaticStyle(staticStyle, options),
      dynamic = _parseStaticStyle.dynamic,
      styleResult = _parseStaticStyle.styleResult;

  if (process.env.NODE_ENV !== 'production' && dynamic) {
    warn('style="' + String(staticStyle) + '": ' + 'Interpolation inside attributes has been deprecated. ' + 'Use v-bind or the colon shorthand instead.');
  }
  if (!dynamic && styleResult) {
    el.staticStyle = styleResult;
  }
  var styleBinding = (0, _helpers.getBindingAttr)(el, 'style', false /* getStatic */);
  if (styleBinding) {
    el.styleBinding = styleBinding;
  } else if (dynamic) {
    el.styleBinding = styleResult;
  }
}

function genData(el) {
  var data = '';
  if (el.staticStyle) {
    data += 'staticStyle:' + el.staticStyle + ',';
  }
  if (el.styleBinding) {
    data += 'style:' + el.styleBinding + ',';
  }
  return data;
}

function parseStaticStyle(staticStyle, options) {
  // "width: 200px; height: 200px;" -> {width: 200, height: 200}
  // "width: 200px; height: {{y}}" -> {width: 200, height: y}
  var dynamic = false;
  var styleResult = '';
  if (staticStyle) {
    var styleList = staticStyle.trim().split(';').map(function (style) {
      var result = style.trim().split(':');
      if (result.length !== 2) {
        return;
      }
      var key = normalize(result[0].trim());
      var value = result[1].trim();
      var dynamicValue = (0, _textParser.parseText)(value, options.delimiters);
      if (dynamicValue) {
        dynamic = true;
        return key + ':' + dynamicValue;
      }
      return key + ':' + JSON.stringify(value);
    }).filter(function (result) {
      return result;
    });
    if (styleList.length) {
      styleResult = '{' + styleList.join(',') + '}';
    }
  }
  return { dynamic: dynamic, styleResult: styleResult };
}

exports.default = {
  staticKeys: ['staticStyle'],
  transformNode: transformNode,
  genData: genData
};