'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createRenderFunction = createRenderFunction;

var _util = require('vue-template-compiler/lib/shared/util');

var _util2 = require('vue-template-compiler/lib/platforms/web/server/util');

var _constants = require('vue-template-compiler/lib/shared/constants');

var _renderContext = require('./render-context');

var _compiler = require('vue-template-compiler/lib/platforms/web/server/compiler');

var _runtimeHelpers = require('./optimizing-compiler/runtime-helpers');

var _createComponent = require('vue-template-compiler/lib/core/vdom/create-component');

var warned = Object.create(null);

var warnOnce = function warnOnce(msg) {
  if (!warned[msg]) {
    warned[msg] = true;
    console.warn('\n\x1B[31m' + msg + '\x1B[39m\n');
  }
};

var normalizeRender = function normalizeRender(vm) {
  var _vm$$options = vm.$options,
      render = _vm$$options.render,
      template = _vm$$options.template,
      _scopeId = _vm$$options._scopeId;

  if ((0, _util.isUndef)(render)) {
    if (template) {
      Object.assign(vm.$options, (0, _compiler.ssrCompileToFunctions)(template, {
        scopeId: _scopeId
      }));
    } else {
      throw new Error('render function or template not defined in component: ' + (vm.$options.name || vm.$options._componentTag || 'anonymous'));
    }
  }
};

function renderNode(node, isRoot, context) {
  if (node.isString) {
    renderStringNode(node, context);
  } else if ((0, _util.isDef)(node.componentOptions)) {
    renderComponent(node, isRoot, context);
  } else if ((0, _util.isDef)(node.tag)) {
    renderElement(node, isRoot, context);
  } else if ((0, _util.isTrue)(node.isComment)) {
    if ((0, _util.isDef)(node.asyncFactory)) {
      // async component
      renderAsyncComponent(node, isRoot, context);
    } else {
      context.write('<!--' + node.text + '-->', context.next);
    }
  } else {
    context.write(node.raw ? node.text : (0, _util2.escape)(String(node.text)), context.next);
  }
}

function registerComponentForCache(options, write) {
  // exposed by vue-loader, need to call this if cache hit because
  // component lifecycle hooks will not be called.
  var register = options._ssrRegister;
  if (write.caching && (0, _util.isDef)(register)) {
    write.componentBuffer[write.componentBuffer.length - 1].add(register);
  }
  return register;
}

function renderComponent(node, isRoot, context) {
  var write = context.write,
      next = context.next,
      userContext = context.userContext;

  // check cache hit

  var Ctor = node.componentOptions.Ctor;
  var getKey = Ctor.options.serverCacheKey;
  var name = Ctor.options.name;
  var cache = context.cache;
  var registerComponent = registerComponentForCache(Ctor.options, write);

  if ((0, _util.isDef)(getKey) && (0, _util.isDef)(cache) && (0, _util.isDef)(name)) {
    var key = name + '::' + getKey(node.componentOptions.propsData);
    var has = context.has,
        get = context.get;

    if ((0, _util.isDef)(has)) {
      has(key, function (hit) {
        if (hit === true && (0, _util.isDef)(get)) {
          get(key, function (res) {
            if ((0, _util.isDef)(registerComponent)) {
              registerComponent(userContext);
            }
            res.components.forEach(function (register) {
              return register(userContext);
            });
            write(res.html, next);
          });
        } else {
          renderComponentWithCache(node, isRoot, key, context);
        }
      });
    } else if ((0, _util.isDef)(get)) {
      get(key, function (res) {
        if ((0, _util.isDef)(res)) {
          if ((0, _util.isDef)(registerComponent)) {
            registerComponent(userContext);
          }
          res.components.forEach(function (register) {
            return register(userContext);
          });
          write(res.html, next);
        } else {
          renderComponentWithCache(node, isRoot, key, context);
        }
      });
    }
  } else {
    if ((0, _util.isDef)(getKey) && (0, _util.isUndef)(cache)) {
      warnOnce('[vue-server-renderer] Component ' + (Ctor.options.name || '(anonymous)') + ' implemented serverCacheKey, ' + 'but no cache was provided to the renderer.');
    }
    if ((0, _util.isDef)(getKey) && (0, _util.isUndef)(name)) {
      warnOnce('[vue-server-renderer] Components that implement "serverCacheKey" ' + 'must also define a unique "name" option.');
    }
    renderComponentInner(node, isRoot, context);
  }
}

function renderComponentWithCache(node, isRoot, key, context) {
  var write = context.write;
  write.caching = true;
  var buffer = write.cacheBuffer;
  var bufferIndex = buffer.push('') - 1;
  var componentBuffer = write.componentBuffer;
  componentBuffer.push(new Set());
  context.renderStates.push({
    type: 'ComponentWithCache',
    key: key,
    buffer: buffer,
    bufferIndex: bufferIndex,
    componentBuffer: componentBuffer
  });
  renderComponentInner(node, isRoot, context);
}

function renderComponentInner(node, isRoot, context) {
  var prevActive = context.activeInstance;
  // expose userContext on vnode
  node.ssrContext = context.userContext;
  var child = context.activeInstance = (0, _createComponent.createComponentInstanceForVnode)(node, context.activeInstance);
  normalizeRender(child);
  var childNode = child._render();
  childNode.parent = node;
  context.renderStates.push({
    type: 'Component',
    prevActive: prevActive
  });
  renderNode(childNode, isRoot, context);
}

function renderAsyncComponent(node, isRoot, context) {
  var factory = node.asyncFactory;

  var resolve = function resolve(comp) {
    if (comp.__esModule && comp.default) {
      comp = comp.default;
    }
    var _node$asyncMeta = node.asyncMeta,
        data = _node$asyncMeta.data,
        children = _node$asyncMeta.children,
        tag = _node$asyncMeta.tag;

    var nodeContext = node.asyncMeta.context;
    var resolvedNode = (0, _createComponent.createComponent)(comp, data, nodeContext, children, tag);
    if (resolvedNode) {
      renderComponent(resolvedNode, isRoot, context);
    } else {
      reject();
    }
  };

  var reject = function reject(err) {
    console.error('[vue-server-renderer] error when rendering async component:\n');
    if (err) console.error(err.stack);
    context.write('<!--' + node.text + '-->', context.next);
  };

  if (factory.resolved) {
    resolve(factory.resolved);
    return;
  }

  var res = void 0;
  try {
    res = factory(resolve, reject);
  } catch (e) {
    reject(e);
  }
  if (res) {
    if (typeof res.then === 'function') {
      res.then(resolve, reject).catch(reject);
    } else {
      // new syntax in 2.3
      var comp = res.component;
      if (comp && typeof comp.then === 'function') {
        comp.then(resolve, reject).catch(reject);
      }
    }
  }
}

function renderStringNode(el, context) {
  var write = context.write,
      next = context.next;

  if ((0, _util.isUndef)(el.children) || el.children.length === 0) {
    write(el.open + (el.close || ''), next);
  } else {
    var children = el.children;
    context.renderStates.push({
      type: 'Element',
      rendered: 0,
      total: children.length,
      endTag: el.close, children: children
    });
    write(el.open, next);
  }
}

function renderElement(el, isRoot, context) {
  var write = context.write,
      next = context.next;


  if ((0, _util.isTrue)(isRoot)) {
    if (!el.data) el.data = {};
    if (!el.data.attrs) el.data.attrs = {};
    el.data.attrs[_constants.SSR_ATTR] = 'true';
  }

  if (el.functionalOptions) {
    registerComponentForCache(el.functionalOptions, write);
  }

  var startTag = renderStartingTag(el, context);
  var endTag = '</' + el.tag + '>';
  if (context.isUnaryTag(el.tag)) {
    write(startTag, next);
  } else if ((0, _util.isUndef)(el.children) || el.children.length === 0) {
    write(startTag + endTag, next);
  } else {
    var children = el.children;
    context.renderStates.push({
      type: 'Element',
      rendered: 0,
      total: children.length,
      endTag: endTag, children: children
    });
    write(startTag, next);
  }
}

function hasAncestorData(node) {
  var parentNode = node.parent;
  return (0, _util.isDef)(parentNode) && ((0, _util.isDef)(parentNode.data) || hasAncestorData(parentNode));
}

function getVShowDirectiveInfo(node) {
  var dir = void 0;
  var tmp = void 0;

  while ((0, _util.isDef)(node)) {
    if (node.data && node.data.directives) {
      tmp = node.data.directives.find(function (dir) {
        return dir.name === 'show';
      });
      if (tmp) {
        dir = tmp;
      }
    }
    node = node.parent;
  }
  return dir;
}

function renderStartingTag(node, context) {
  var markup = '<' + node.tag;
  var directives = context.directives,
      modules = context.modules;

  // construct synthetic data for module processing
  // because modules like style also produce code by parent VNode data

  if ((0, _util.isUndef)(node.data) && hasAncestorData(node)) {
    node.data = {};
  }
  if ((0, _util.isDef)(node.data)) {
    // check directives
    var dirs = node.data.directives;
    if (dirs) {
      for (var i = 0; i < dirs.length; i++) {
        var name = dirs[i].name;
        var dirRenderer = directives[name];
        if (dirRenderer && name !== 'show') {
          // directives mutate the node's data
          // which then gets rendered by modules
          dirRenderer(node, dirs[i]);
        }
      }
    }

    // v-show directive needs to be merged from parent to child
    var vshowDirectiveInfo = getVShowDirectiveInfo(node);
    if (vshowDirectiveInfo) {
      directives.show(node, vshowDirectiveInfo);
    }

    // apply other modules
    for (var _i = 0; _i < modules.length; _i++) {
      var res = modules[_i](node);
      if (res) {
        markup += res;
      }
    }
  }
  // attach scoped CSS ID
  var scopeId = void 0;
  var activeInstance = context.activeInstance;
  if ((0, _util.isDef)(activeInstance) && activeInstance !== node.context && (0, _util.isDef)(scopeId = activeInstance.$options._scopeId)) {
    markup += ' ' + scopeId;
  }
  while ((0, _util.isDef)(node)) {
    if ((0, _util.isDef)(scopeId = node.context.$options._scopeId)) {
      markup += ' ' + scopeId;
    }
    node = node.parent;
  }
  return markup + '>';
}

function createRenderFunction(modules, directives, isUnaryTag, cache) {
  return function render(component, write, userContext, done) {
    warned = Object.create(null);
    var context = new _renderContext.RenderContext({
      activeInstance: component,
      userContext: userContext,
      write: write, done: done, renderNode: renderNode,
      isUnaryTag: isUnaryTag, modules: modules, directives: directives,
      cache: cache
    });
    (0, _runtimeHelpers.installSSRHelpers)(component);
    normalizeRender(component);
    renderNode(component._render(), true, context);
  };
}